/**
 *
 * \file     kml_context.h
 * \brief    structure of KMATHLIB API context
 * \authoers Toshiyuki Imamura (TI)
 * \date     2014/12/23 (NT)
 * 
 * (c) Copyright 2014 RIKEN. All rights reserved.
 */

#ifndef __kml_context_h__
#define __kml_context_h__

#include "kml_hash_table.h"

typedef void (*Function1)(long long* context, long long* pcontext, int* error);

typedef void (*Function2)(long long* context, long long* pcontext, int* error,
						  int* param, long long* petsc);

typedef void (*Function3)(long long* context, long long* pcontext, int* error,
						  int* param_out, int* param_in);

#define MAX_VARIABLES 16
#define MAX_VAR_LEN   16

typedef struct {

  int        comm;

  kml_list*  parameters[HASH_SIZE];
  kml_list*  in_para_vars [HASH_SIZE];
  kml_list*  out_para_vars[HASH_SIZE];

  char       in_variables [MAX_VARIABLES][MAX_VAR_LEN];
  char       out_variables[MAX_VARIABLES][MAX_VAR_LEN];
  char       mem_variables[MAX_VARIABLES][MAX_VAR_LEN];
  long long* plugin_context;
  Function1  func_initialize;
  Function1  func_shutdown;
  Function1  func_realloc;
  Function1  func_solve;
  Function2  func_to_internal;
  Function2  func_to_standard;
  Function3  func_reuse_internal;

} kml_context;

#endif // !__kml_context_h__
