/**
 *
 * \file     kml_plugin_reg.c
 * \brief    kmath lib API plugin module register routine
 * \authoers Toshiyuki Imamura (TI)
 * \date     2014/12/24 (NT)
 * 
 * (c) Copyright 2014 RIKEN. All rights reserved.
 */

#include "kml_plugin_if.h"
#include "kml_context.h"
#include "kml_error_code.h"
#include <string.h>
#include <stdio.h>

void
kml_regist_plugin_routines_(long long* handle, 
							long long* pcontext,
							const char* in_variables,
							const char* out_variables,
							const char* mem_variables,
							void* func_initialize,
							void* func_shutdown,
							void* func_realloc,
							void* func_solve,
							void* func_to_internal,
							void* func_to_standard,
							void* func_reuse_internal,
							int in_param_len,
							int out_param_len,
							int mem_param_len)
{
  kml_context* c = (kml_context *)*handle;

  int i;
  char params[256], *tok;

  memset(c->in_variables,  0, sizeof(char) * MAX_VARIABLES * MAX_VAR_LEN);
  memset(c->out_variables, 0, sizeof(char) * MAX_VARIABLES * MAX_VAR_LEN);
  memset(c->mem_variables, 0, sizeof(char) * MAX_VARIABLES * MAX_VAR_LEN);

  // check input parameters
  strncpy(params, in_variables, in_param_len);
  params[in_param_len] = 0;

  tok = strtok(params, ",");
  for(i = 0; i < MAX_VARIABLES; ++i)
	{
	  if (tok != NULL)
		strcpy(c->in_variables[i], tok);
	  else
		break;
	  tok = strtok(NULL, ",");
	}

  // check output parameters
  strncpy(params, out_variables, out_param_len);
  params[out_param_len] = 0;

  tok = strtok(params, ",");
  for(i = 0; i < MAX_VARIABLES; ++i)
	{
	  if (tok != NULL)
		strcpy(c->out_variables[i], tok);
	  else
		break;
	  tok = strtok(NULL, ",");
	}

  // check memory allocation parameters
  strncpy(params, mem_variables, mem_param_len);
  params[mem_param_len] = 0;

  tok = strtok(params, ",");
  for(i = 0; i < MAX_VARIABLES; ++i)
	{
	  if (tok != NULL)
		strcpy(c->mem_variables[i], tok);
	  else
		break;
	  tok = strtok(NULL, ",");
	}

  c->plugin_context      = pcontext;
  c->func_initialize     = (Function1)func_initialize;
  c->func_shutdown       = (Function1)func_shutdown;
  c->func_solve          = (Function1)func_solve;
  c->func_realloc        = (Function1)func_realloc;
  c->func_to_internal    = (Function2)func_to_internal;
  c->func_to_standard    = (Function2)func_to_standard;
  c->func_reuse_internal = (Function3)func_reuse_internal;
}

#define CHECK_ERROR(__type)			\
  if (kval == NULL)					\
	{								\
	  *err = EI_ParamNotFound;		\
	  return ;						\
	}								\
  if (kval[0] != __type)			\
	{								\
	  *err = EI_FatalGetParam;		\
	  return ;						\
	}

void
kml_get_mpi_communicator_(long long* handle, int* comm)
{
  kml_context* c = (kml_context *)*handle;

  *comm = c->comm;
}

void
kml_get_parameter_int_(long long* handle,
					   const char* key, int* val, int* err,
					   int key_len)
{
  kml_context* c = (kml_context *)*handle;

  char ckey[64];
  strncpy(ckey, key, key_len);
  ckey[key_len] = 0;

  const char* kval = kml_hash_table_get(c->parameters, ckey);

  CHECK_ERROR('I');

  sscanf(kval+1, "%d", val);
  *err = EI_None;
}

void
kml_get_parameter_real_(long long* handle,
						const char* key, double* val, int* err,
						int key_len)
{
  kml_context* c = (kml_context *)*handle;

  char ckey[64];
  strncpy(ckey, key, key_len);
  ckey[key_len] = 0;

  const char* kval = kml_hash_table_get(c->parameters, ckey);

  CHECK_ERROR('R');

  sscanf(kval+1, "%lf", val);
  *err = EI_None;
}

void
kml_get_parameter_string_(long long* handle,
						  const char* key, char* val, int* err, 
						  int key_len, int val_len)
{
  kml_context* c = (kml_context *)*handle;

  char ckey[64];
  strncpy(ckey, key, key_len);
  ckey[key_len] = 0;

  const char* kval = kml_hash_table_get(c->parameters, ckey);

  CHECK_ERROR('S');

  memset(val, ' ', sizeof(char)*val_len);
  sscanf(kval+1, "%s", val);
  val[strlen(val)] = ' ';
  *err = EI_None;
}

void
kml_get_parameter_petsc_(long long* handle,
						 const char* key, long long* val, int* err,
						 int key_len)
{
  kml_context* c = (kml_context *)*handle;

  char ckey[64];
  strncpy(ckey, key, key_len);
  ckey[key_len] = 0;

  const char* kval = kml_hash_table_get(c->parameters, ckey);

  CHECK_ERROR('P');

  sscanf(kval+1, "%lld", val);
  *err = EI_None;
}
