!--------1---------2---------3---------4---------5---------6---------7---------8
!
!  Module   kml_plugin_tools_mod
!> @brief   tools for kmath lib API plugin module
!! @authors Toshiyuki Imamura (TI)
!! @date    2015/02/09 (NT)
!
!  (c) Copyright 2014 RIKEN. All rights reserved.
!
!--------1---------2---------3---------4---------5---------6---------7---------8

module kml_plugin_tools_mod

  implicit none
  private

#include "kml_error_code.h"

  ! public procedures
  !
  public  :: kml_get_row_col
  public  :: kml_get_prime_factor
  public  :: kml_tolower_case
  public  :: kml_set_error

contains

  !======1=========2=========3=========4=========5=========6=========7=========8
  !
  !  Subroutine  kml_get_row_col
  !> @brief      get number of row and number of col
  !! @authors    NT
  !! @date       2015/02/16 (NT)
  !
  !======1=========2=========3=========4=========5=========6=========7=========8
  
  subroutine kml_get_row_col(nprocs, nrow, ncol)

    ! formal arguments
    integer,                 intent(in)    :: nprocs
    integer,                 intent(out)   :: nrow
    integer,                 intent(out)   :: ncol

    ! local variables
    integer                  :: k


    ncol = int(sqrt(dble(nprocs)))

    k = 1
    do
      if (ncol <= k) &
        exit
      if (mod(ncol,k) == 0 .and. mod(nprocs, ncol) == 0) exit
      ncol = ncol-1
    end do
    nrow = nprocs/ncol

    return

  end subroutine kml_get_row_col

  !======1=========2=========3=========4=========5=========6=========7=========8
  !
  !  Subroutine  kml_prime_factor
  !> @brief      get prime factor
  !! @authors    NT
  !! @date       2015/02/09 (NT)
  !
  !======1=========2=========3=========4=========5=========6=========7=========8

  subroutine kml_get_prime_factor(val, factors)

    ! formal arguments
    integer,                 intent(in)    :: val
    integer,                 intent(inout) :: factors(:)

    ! local variables
    integer                  :: v, pf, i, rmax


    v = val
    rmax = int(sqrt(real(v)))

    factors(:) = 0
    i = 0

    do pf = 2, rmax
      do while (mod(v, pf) == 0)
        i = i + 1
        factors(i) = pf
        v = v / pf
      end do
      if (v < pf) &
        exit
    end do
    if (i == 0 .or. v > 1) then
      i = i + 1
      factors(i) = v
    end if

    return

  end subroutine kml_get_prime_factor

  !======1=========2=========3=========4=========5=========6=========7=========8
  !
  !  Subroutine  kml_tolower_case
  !> @brief      convert to lower case string
  !! @authors    NT
  !! @date       2015/02/09 (NT)
  !
  !======1=========2=========3=========4=========5=========6=========7=========8

  subroutine kml_tolower_case(str)

    ! formal arguments
    character(*),            intent(inout) :: str

    ! local variables
    integer                  :: i


    do i = 1, len(str)
      if (str(i:i) >= 'A' .and. str(i:i) <= 'Z') then
        str(i:i) = char(ichar(str(i:i)) + 32)
      end if
    end do

    return

  end subroutine kml_tolower_case

  !======1=========2=========3=========4=========5=========6=========7=========8
  !
  !  Subroutine  kml_set_error
  !> @brief      set up error code
  !! @authors    NT
  !! @date       2015/03/10 (NT)
  !
  !======1=========2=========3=========4=========5=========6=========7=========8

  subroutine kml_set_error(error, code)

    ! formal arguments
    integer,                 intent(inout) :: error
    integer,                 intent(in)    :: code

#if defined(DEBUG)
    ! parameters
    character(*), parameter  :: CodeString(8) = (/'EP_SolveFail      ', &
                                                  'EP_FatalInit      ', &
                                                  'EP_FatalShutdown  ', &
                                                  'EP_FatalToInternal', &
                                                  'EP_FatalToStandard', &
                                                  'EP_FatalRealloc   ', &
                                                  'EP_FatalReuse     ', &
                                                  'EP_FatalSolve     '/)

    if (EP_SolveFail >= code .and. code >= EP_FatalSolve) then
      write(6,*) 'Kml_Set_Error> [', CodeString(abs(code-EP_SolveFail)+ 1), ']'
    else if (code /= EP_None) then
      write(6,*) 'Kml_Set_Error> unkown code : ', code
    end if
#endif

    error = code

    return

  end subroutine kml_set_error
  
end module kml_plugin_tools_mod
