program test

  use kmath_lib_mod
  use kmath_plugin_sls_petsc_mod

  implicit none

#include <finclude/petscsys.h>
#include <finclude/petscvec.h>
#include <finclude/petscmat.h>
#include <finclude/petscviewer.h>


  PetscViewer    :: view
  PetscRandom    :: r
  Vec            ::  x, b, y
  Mat            ::  A

  PetscReal      :: norm
  PetscInt       :: mDim, nDim, ierr
  PetscBool      :: flg
  PetscMPIInt    :: size, rank

  character(128) ::  fin

  type(s_context) :: h


  call PetscInitialize(PETSC_NULL_CHARACTER,ierr)
  call MPI_Comm_size(PETSC_COMM_WORLD,size,ierr)
  call MPI_Comm_rank(PETSC_COMM_WORLD,rank,ierr)

  ! read binary matrix file
  call PetscOptionsGetString(PETSC_NULL_CHARACTER,'-f',fin,flg,ierr)
  call PetscViewerBinaryOpen(PETSC_COMM_WORLD,fin,FILE_MODE_READ,view,ierr)
  call MatCreate(PETSC_COMM_WORLD,A,ierr)
  call MatSetType(A,MATAIJ,ierr)
  call MatLoad(A,view,ierr)
  call PetscViewerDestroy(view,ierr)
  call MatGetSize(A, mDim, nDim, ierr)

  if (rank == 0) then
    print*,'Mat Size = ', mDim, nDim
  end if

  ! create right-hand-side vector. (random number)
  call VecCreate(PETSC_COMM_WORLD,b,ierr)
  call VecSetSizes(b,PETSC_DECIDE,mDim,ierr)
  call VecSetFromOptions(b,ierr)
  call VecDuplicate(b,x,ierr)
  call PetscRandomCreate(PETSC_COMM_SELF,r,ierr)
  call PetscRandomSetFromOptions(r,ierr)
  call VecSetRandom(b,r,ierr)
  call PetscRandomDestroy(r,ierr)

  call VecDuplicate(x, y, ierr)

  ! solve by KMATH API
  call KMATH_Create(h, PETSC_COMM_WORLD, ierr)
  call KMATH_Set_Plugin(h, KMATH_Plugin_Setup_SLS_PETSc, ierr)

  call KMATH_Set_Parameter(h, 'A', A, ierr)
  call KMATH_Set_Parameter(h, 'B', b, ierr)
  call KMATH_Set_Parameter(h, 'X', x, ierr)
  call KMATH_Solve(h, ierr)

  call KMATH_Destroy(h, ierr)

  ! check result
  call MatMult(A, x, y, ierr)
  call VecAXPY(y, -1.0d0, b, ierr)
  call VecNorm(b, NORM_2, norm, ierr)
  if(rank == 0) then
    print*,'rhs-b(2NORM) = ',norm
  end if

  call VecNorm(y, NORM_2, norm, ierr)
  if(rank == 0) then
    print*,'diff(2NORM) = ',norm
  end if

  ! clean up
  call VecDestroy(y, ierr)
  call VecDestroy(x, ierr)
  call VecDestroy(b, ierr)
  call MatDestroy(A, ierr)

  call PetscFinalize(ierr)

  stop

end program test
