program test

  use kmath_lib_mod
  use kmath_plugin_deig_scalapack_mod
  use kmath_plugin_deig_eigenexa_mod

  implicit none

#include <finclude/petscsys.h>
#include <finclude/petscvec.h>
#include <finclude/petscmat.h>

  Mat :: A
  Mat :: Z1
  Mat :: Z2
  Vec :: X1
  Vec :: X2

  PetscInt             :: ierr
  PetscScalar          :: v(1)
  PetscScalar          :: r1
  PetscReal            :: norm
  PetscReal            :: r
  integer              :: sn
  integer              :: rank
  integer              :: m, n, i, j, iar(1), jar(1)
  integer, allocatable :: seed(:)
  type(s_context)      :: h


  call random_seed(size = sn)
  allocate(seed(sn))
  seed = 123;
  call random_seed(put = seed)
  deallocate(seed)

  call PetscInitialize(PETSC_NULL_CHARACTER, ierr)

  call MPI_Comm_rank(PETSC_COMM_WORLD,rank,ierr)

  ! create matrix & vector
  m = 1000
  n = 1000

  call MatCreateDense(PETSC_COMM_WORLD, PETSC_DECIDE, &
       PETSC_DECIDE, m, n, PETSC_NULL_SCALAR,A, ierr)

  call VecCreate     (PETSC_COMM_WORLD, X1, ierr)
  call VecSetType    (X1, VECMPI, ierr)
  call VecSetSizes   (X1, PETSC_DECIDE, n, ierr)

  call MatCreateDense(PETSC_COMM_WORLD, PETSC_DECIDE, &
       PETSC_DECIDE, m, n, PETSC_NULL_SCALAR, Z1, ierr)

  call VecCreate     (PETSC_COMM_WORLD, X2, ierr)
  call VecSetType    (X2, VECMPI, ierr)
  call VecSetSizes   (X2, PETSC_DECIDE, n, ierr)

  call MatCreateDense(PETSC_COMM_WORLD, PETSC_DECIDE, &
       PETSC_DECIDE, m, n, PETSC_NULL_SCALAR, Z2, ierr)

  ! set symmetric matrix
  if (rank == 0) then
    do i = 0, m - 1
      iar(1) = i
      do j = 0, i
        jar(1) = j
        call random_number(r)
        r1 = r
        v(1) = r1
        call MatSetValues(A, 1, iar, 1, jar, v, INSERT_VALUES, ierr)
        if(i /= j) &
          call MatSetValues(A, 1, jar, 1, iar, v, INSERT_VALUES, ierr)
      end do
    end do
  end if

  call MatAssemblyBegin(A,MAT_FINAL_ASSEMBLY,ierr)
  call MatAssemblyEnd  (A,MAT_FINAL_ASSEMBLY,ierr)

  ! solve by KMATH API (with ScaLAPACK)
  call KMATH_Create(h, PETSC_COMM_WORLD, ierr)

  call KMATH_Set_Plugin(h, KMATH_Plugin_Setup_DEIG_ScaLAPACK, ierr)
  call KMATH_Set_Parameter(h, 'A', A,  ierr)
  call KMATH_Set_Parameter(h, 'X', X1, ierr)
  call KMATH_Set_Parameter(h, 'Z', Z1, ierr)
  call KMATH_Set_Parameter(h, 'matrix_type', 'hermitian', ierr)
  call KMATH_Solve(h, ierr)

  ! solve by KMATH API (with Eigen-Exa)
  call KMATH_Set_Plugin(h, KMATH_Plugin_Setup_DEIG_EigenExa, ierr)
  call KMATH_Set_Parameter(h, 'A', A,  ierr)
  call KMATH_Set_Parameter(h, 'X', X2, ierr)
  call KMATH_Set_Parameter(h, 'Z', Z2, ierr)
  call KMATH_Solve(h, ierr)

  call KMATH_Destroy(h, ierr)

  ! result check
  call VecAXPY(X1, -1.0d0, X2, ierr)
  call VecNorm(X1, NORM_2, norm, ierr)

  if(rank == 0) then
    print*,'diff(2NORM) = ', norm
  end if

  call MatDestroy(A, ierr)
  call VecDestroy(X1,ierr)
  call MatDestroy(Z1,ierr)
  call VecDestroy(X2,ierr)
  call MatDestroy(Z2,ierr)

  call PetscFinalize(ierr)

end program test
