program test

  use kmath_lib_mod
  use kmath_plugin_dls_scalapack_mod

  implicit none

#include <finclude/petscsys.h>
#include <finclude/petscvec.h>
#include <finclude/petscmat.h>

  Mat :: A
  Vec :: B
  Vec :: X
  Vec :: Y

  PetscInt             :: ierr
  PetscScalar          :: v(1)
  PetscScalar          :: r1
  PetscReal            :: norm
  PetscReal            :: r
  integer              :: sn
  integer              :: rank
  integer              :: m, n, i, j, iar(1), jar(1)
  integer, allocatable :: seed(:)
  type(s_context)      :: h


  call random_seed(size = sn)
  allocate(seed(sn))
  seed = 123;
  call random_seed(put = seed)
  deallocate(seed)

  call PetscInitialize(PETSC_NULL_CHARACTER, ierr)

  call MPI_Comm_rank(PETSC_COMM_WORLD,rank,ierr)

  ! create matrix & vector
  m = 2000
  n = 2000

  call MatCreateDense(PETSC_COMM_WORLD, PETSC_DECIDE, &
       PETSC_DECIDE, m, n, PETSC_NULL_SCALAR,A, ierr)

  call VecCreate(PETSC_COMM_WORLD, B, ierr)
  call VecSetType(B, VECMPI, ierr)
  call VecSetSizes(B, PETSC_DECIDE, n, ierr)

  call VecCreate(PETSC_COMM_WORLD, X, ierr)
  call VecSetType(X, VECMPI, ierr)
  call VecSetSizes(X, PETSC_DECIDE, n, ierr)

  call VecDuplicate(X, Y, ierr)

  if (rank == 0) then
    do i = 0, m - 1
      iar(1) = i
      do j = 0, n - 1
        jar(1) = j
        call random_number(r)
        r1 = r
        v(1) = r1
        call MatSetValues(A, 1, iar, 1, jar, v, INSERT_VALUES, ierr)
      end do
    end do
  end if

  call MatAssemblyBegin(A,MAT_FINAL_ASSEMBLY,ierr)
  call MatAssemblyEnd  (A,MAT_FINAL_ASSEMBLY,ierr)

  if (rank == 0) then
    do i = 0, n - 1
      iar(1) = i
      call random_number(r)
      r1 = r
      v(1) = r1
      call VecSetValues(B, 1, iar, v, INSERT_VALUES, ierr)
    end do
  end if

  call VecAssemblyBegin(B, ierr)
  call VecAssemblyEnd  (B, ierr)

  ! solve by KMATH API (with ScaLAPACK)
  call KMATH_Create(h, PETSC_COMM_WORLD, ierr)

  call KMATH_Set_Plugin(h, KMATH_Plugin_Setup_DLS_ScaLAPACK, ierr)
  call KMATH_Set_Parameter(h, 'A', A, ierr)
  call KMATH_Set_Parameter(h, 'B', B, ierr)
  call KMATH_Set_Parameter(h, 'X', X, ierr)
  call KMATH_Solve(h, ierr)

  call KMATH_Destroy(h, ierr)

  ! result check
  call MatMult(A, X, Y, ierr)
  call VecAXPY(Y, -1.0d0, B, ierr)
  call VecNorm(Y, NORM_2, norm, ierr)
  if (rank == 0) then
    print*,'diff(2NORM) = ',norm
  end if

  call VecDestroy(Y, ierr)
  call VecDestroy(X, ierr)
  call VecDestroy(B, ierr)
  call MatDestroy(A, ierr)

  call PetscFinalize(ierr)

end program test
